require 'tmpdir'
require 'webrick'
require 'net/http'
require 'base64'
require 'json'

entrypoint = '__NOW_HANDLER_FILENAME'
host = '0.0.0.0'
port = 3000

def rack_handler
  require 'rack'

  th = Thread.new do
    Rack::Server.start :config => entrypoint, :Host => host, :Port => port
  end

  proc { Thread.kill(th) }
end

def webrick_handler
  require_relative entrypoint

  if not Object.const_defined?('Handler')
    return { :statusCode => 500, :body => 'Handler not defined in lambda' }
  end

  server = WEBrick::HTTPServer.new :BindAddress => host, :Port => port

  if Handler.is_a?(Proc)
    server.mount_proc '/', Handler
  else
    server.mount '/', Handler
  end

  th = Thread.new(server) do |server|
    server.start
  end

  proc {
    server.shutdown
    Thread.kill(th)
  }
end

def now__handler(event:, context:)
  payload = JSON.parse(event['body'])
  path = payload['path']
  headers = payload['headers']
  httpMethod = payload['method']
  encoding = payload['encoding']
  body = payload['body']

  if (not body.nil? and not body.empty?) and (not encoding.nil? and encoding == 'base64')
    body = Base64.decode64(body)
  end

  if entrypoint.ends_with? 'config.ru'
    cleanup = rack_handler
  else
    cleanup = webrick_handler
  end

  Signal.list.keys.each do |sig|
    begin
      Signal.trap(sig, cleanup)
    rescue
    end
  end

  http = Net::HTTP.new(host, port)
  res = http.send_request(httpMethod, path, body, headers)

  cleanup.call

  {
    :statusCode => res.code.to_i,
    :headers => res.each_capitalized.to_h,
    :body => res.body
  }
end
